# osctl 8                         2018-01-03                              0.1.0

## NAME
`osctl` - command line interface for `osctld`, the management daemon from
vpsAdminOS.

## SYNOPSIS
`osctl` [*global options*] *command* [*command options*] [*arguments...*]

## DESCRIPTION
`osctl` is a command line interface for `osctld`. `osctld` is a daemon from
vpsAdminOS that is used to manage unprivileged Linux containers, including
storage pools, user namespaces and cgroups for resource management.
`osctld` must be running before `osctl` can be used. `osctl` is available only
to root.

## POOLS
`osctld` fully utilizes ZFS and currently does not support any other file system.
`osctld` stores its data and configuration on provided zpools, no other path
in the system has to be persistent.

`osctld` needs at least one zpool to operate, see `pool install`
and `pool import`. When managing entities such as groups or containers
with multiple pools, you may need to specify pool name if there are name
conflicts, e.g. two groups or containers from different pools with the same
name. Two users with the same name are not allowed, because of system user/group
conflict. `osctld` by default selects the entity from the first pool that has
it. If you wish to manage such entities from other pools, you can use global
option `--pool` *name* or specify the group/container name/id as *pool*:*name*,
i.e. pool name and group/container name/id separated by colon.

## USER NAMESPACES
`osctld` makes it possible to run every container within a different user
namespace. It makes it possible by managing system users that are used to spawn
unprivileged containers. `osctld` manages all important system files, such as
**/etc/passwd**, **/etc/group**, **/etc/subuid**, **/etc/subgid** or
**/etc/lxc/lxc-usernet**. See the `user` commands.

## GROUPS
Groups represent the cgroup hierarchy. It is possible to define groups that
`osctld` will manage and configure their parameters. There are always at least
two groups: **root** and **default**. **root** group is the parent of all
managed groups and **default** is the group that containers are placed in unless
configured otherwise. Every container belongs to exactly one group.

See the `group` commands.

## CONTAINERS
`osctld` utilizes LXC to setup and run containers. Each container can belong
to a different user namespace and resource group. `osctld` set's up the
container's environment and LXC homes.

See the `ct` commands.

## CONTAINER MIGRATIONS
`osctld` has support for migrating containers between different vpsAdminOS nodes
with SSH used as a transport channel. Each vpsAdminOS node has a system user
called `migration`. The source node is connecting to the `migration` user on
the destination node. Authentication is based on public/private keys.

On the source node, a public/private key pair is needed. It can be generated by
`migration key gen`, or the keys can be manually installed to paths given by
`migration key path public` and `migration key path private`. Through another
communication channel, picked at your discretion, the public key of the source
node must be transfered to the destination node and authorized to migrate
containers to that node. Once transfered, the key can be authorized using
`migration authorized-keys add`.

The container migration itself consists of several steps:

 - `ct migrate stage` is used to prepare environment on the destination node
   and copy configuration
 - `ct migrate sync` sends over the container's rootfs
 - `ct migrate transfer` stops the container on the source node, performs
   another rootfs sync and finally starts the container on the destination node
 - `ct migrate cleanup` is used to remove the container from the source node

Up until `ct migrate transfer`, the migration can be cancelled using
`ct migrate cancel`.

`ct migrate now` will perform all necessary migration steps in succession.

## GLOBAL OPTIONS
`--help`
  Show help message

`-j`, `--json`
  Format output in JSON.

`-p`, `--parsable`
  Show precise values, useful for parsing in scripts.

`--[no-]color`
  Toggle colorized output for commands that support it. Enabled by default.

`-q`, `--quiet`
  Surpress output.

`--pool=arg`
  Pool name. Needed only when there is more than one pool installed and you wish
  to work with pools other than the first one, which is taken as the default
  one.

`--version`
  Display the program version and exit.

## COMMANDS
`pool install` [*options*] *name*
  Mark zpool *name* to be used with `osctld`.
  User property **org.vpsadminos.osctl:active** is set to **yes**. `osctld` will
  automatically import such marked pools on start. The pool is also immediatelly
  imported, see `pool import`.

    `--dataset` *dataset*
      Scope osctld to *dataset* on zpool *name*. All osctld's data will be stored
      in *dataset*. This option can be useful when the pool is used with other
      applications or data.

`pool uninstall` *name*
  Unmark zpool *name*, i.e. unset the user property set by `pool install`.
  No data is deleted from the pool, it will simply not be automatically imported
  when `osctld` starts.

`pool import` `-a`,`--all`|*name*
  Import zpool *name* into `osctld`. `osctld` will load all users, groups and
  containers from the pool.

    `-a`, `--all`
      Import all installed pools. This is what `osctld` does on start.

    `-s`, `--[no-]autostart`
      Start containers that are configured to be started automatically. Enabled
      by default.

`pool export` [*options*] *name*
  Export pool *name* from `osctld`. No data is deleted, the pool and all its
  content is merely removed from `osctld`. `pool export` aborts if any container
  from the exported pool is running, unless option `-f`, `--force` is given.

    `-f`, `--force`
      Export the pool even if there are containers running or an autostart plan
      is still in progress. Running containers are stopped if `-s`,
      `--stop-containers` is set, otherwise they're left alone.

    `-s`, `--[no-]stop-containers`
      Stop all containers from pool *name*. Enabled by default.

    `-u`, `--[no-]unregister-users`
      Unregister users from pool *name* from the system, i.e. remove entries
      from `/etc/passwd` and `/etc/group`. Enabled by default.

`pool ls` [*names...*]
  List imported pools. If no *names* are provided, all pools are listed.
    
    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output, comma separated.

`pool assets` [*options*] *name*
  List pool assets and their state.

    `-v`, `--verbose`
      Show detected errors.

`pool autostart queue` [*options*] *name*
  Print containers waiting in the auto-start queue. The containers are ordered
  by start priority and id.
    
    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output, comma separated.

`pool autostart trigger` *name*
  Start all containers from pool *name* that are configured to be started
  automatically. This can be used to ensure that all containers that are
  supposed to be running are actually running.

`pool autostart cancel` *name*
  Cancel starting of containers that are configured to be started automatically
  and are left in the start queue.

`user new` *options* *name*
  Create a new user with user namespace configuration.

    `--pool` *name*
      Pool name.
    
    `--ugid` *ugid*
      User/group ID, required.

    `--offset` *offset*
      Offset of user/group IDs from zero used for UID/GID mapping, required.

    `--size` *size*
      Number of user/group IDs available within the user namespace, required.
      Should be 65536 or more.

`user del` *name*
  Delete user *name*.

`user ls` [*options*] [*names...*]
  List available users. If no *names* are provided, all users are listed.

    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output, comma separated.

    `--pool` *names*
      Filter by pool name, comma separated.

    `--registered`
      List only registered users.

    `--unregistered`
      List only unregistered users

`user show` [*options*] *name*
  Show user info.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

`user reg` `all`|*name*
  Register all users or a selected user into the system, i.e. add records
  to **/etc/passwd** and **/etc/group**.

`user unreg` `all`|*name*
  Unregister all users or a selected user from the system, i.e. remove records
  from **/etc/passwd** and **/etc/group**.

`user subugids`
  Generate **/etc/subuid** and **/etc/subgid**.

`user assets` [*options*] *name*
  List user's assets (datasets, files, directories) and their state.

    `-v`, `--verbose`
      Show detected errors.

`ct new` [*options*] *id*
  Create a new container. Selected user and group have to be from the same pool
  the container is being created on.

  Containers are usually created from templates. Templates are by default
  downloaded from remote repositories. The desired template can be selected
  using option `--template` or specified by `--distribution` and optionally
  also `--version`, `--arch`, `--vendor` or `--variant`. All configured
  repositories are searched by default.

  Templates can also be supplied as local files, use option `--from-archive` for
  tar archives, `--from-stream` for ZFS streams.

  Containers can be created in custom datasets, located anywhere on the pool.
  The dataset can be selected using option `--dataset`. If the dataset already
  containers rootfs and you do not wish to use any template, signal this with
  option `--no-template`. Otherwise, the template to be used can be selected
  using any of the methods above.
  
    `--pool` *name*
      Pool name. Defaults to the first available pool.

    `--user` *name*
      User name, required.

    `--group` *name*
      Group name, defaults to group *default* from selected *pool*.

    `--template` *name*
      Template tar file, required. See `TEMPLATE NAMES` for the naming scheme.

    `--from-archive` *file*
      Create the container from a tar archive available in the filesystem.

    `--from-stream` *file*
      Create the container from a ZFS stream, either in a file available in the
      filesystem, or the stream can be fed into `osctl` via standard input.

      If *file* is `-`, the stream is read from the standard input. In this case,
      `--distribution` and `--version` have to be provided.

    `--dataset` *dataset*
      Use a custom dataset for the container's rootfs. The dataset and all its
      parents are created, if it doesn't already exist. If used without
      `--template`, the dataset is expected to already contain the rootfs
      and `--distribution` and `--version` have to be provided.

    `--distribution` *distribution*
      Distribution name in lower case, e.g. alpine, centos, debian, ubuntu.
      If `--template` is provided, this option is not necessary, but can
      optionally override the template's distribution info.

    `--version` *version*
      Distribution version. The format can differ among distributions, e.g.
      alpine `3.6`, centos `7.0`, debian `9.0` or ubuntu `16.04`.
      If `--template` is provided, this option is not necessary, but can
      optionally override the template's distribution version info.

    `--arch` *arch*
      Container architecture, e.g. `x86_64` or `x86`. Defaults to the host system
      architecture.

    `--vendor` *vendor*
      Vendor to be selected from the remote template repository.

    `--variant` *variant*
      Vendor variant to be selected from the remote template repository.

    `--repository` *name*
      Instead of searching all configured repositories from appropriate pool,
      use only repository *name*. The selected repository can be disabled.

`ct del` *id*
  Stop and delete container *id*.

`ct reinstall` [*options*] *id*
  Reinstall container from template. The container's rootfs is deleted
  and an OS template is applied again. The container's configuration
  and subdatasets remain unaffected.

  If no template info is given via command-line options, `osctld` will attempt
  to find appropriate template for the container's distribution version in
  remote repositories. This may not work if the container was created
  from a local file, stream or if the distribution is too old and no longer
  supported.

  As the applied template can be a ZFS stream, it is necessary to delete all
  snapshots of the container's root dataset. By default, `ct reinstall` will
  abort if there are snapshots present. You can use option `-r`,
  `--remove-snapshots` to remove them.

    `--template` *name*
      Template tar file, required. See `TEMPLATE NAMES` for the naming scheme.

    `--from-archive` *file*
      Create the container from a tar archive available in the filesystem.

    `--from-stream` *file*
      Create the container from a ZFS stream, either in a file available in the
      filesystem, or the stream can be fed into `osctl` via standard input.

      If *file* is `-`, the stream is read from the standard input. In this case,
      `--distribution` and `--version` have to be provided.

    `--distribution` *distribution*
      Distribution name in lower case, e.g. alpine, centos, debian, ubuntu.
      If `--template` is provided, this option is not necessary, but can
      optionally override the template's distribution info.

    `--version` *version*
      Distribution version. The format can differ among distributions, e.g.
      alpine `3.6`, centos `7.0`, debian `9.0` or ubuntu `16.04`.
      If `--template` is provided, this option is not necessary, but can
      optionally override the template's distribution version info.

    `--arch` *arch*
      Container architecture, e.g. `x86_64` or `x86`. Defaults to the host system
      architecture.

    `--vendor` *vendor*
      Vendor to be selected from the remote template repository.

    `--variant` *variant*
      Vendor variant to be selected from the remote template repository.

    `--repository` *name*
      Instead of searching all configured repositories from appropriate pool,
      use only repository *name*. The selected repository can be disabled.

    `-r`, `--remove-snapshots`
      Remove all snapshots of the container's root dataset. `ct reinstall`
      cannot proceed if there are snapshots present.

`ct ls` [*options*] [*ids...*]
  List containers. If no *ids* are provided, all containers matching filters
  are listed.

    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output, comma separated. Defaults to a selected
      subset of available parameters.

    `--pool` *names*
      Filter by pool, comma separated.

    `-u`, `--user` *users*
      Filter by user name, comma separated.

    `-g`, `--group` *groups*
      Filter by group name, comma separated.

    `-s`, `--state` *states*
      Filter by state, comma separated. Available states:
      **stopped**, **starting**, **running**, **stopping**, **aborting**, **freezing**,
      **frozen**, **thawed**.

    `-d`, `--distribution` *distributions*
      Filter by distribution, comma separated.

    `-v`, `--version` *versions*
      Filter by distribution version, comma separated.

`ct tree` *pool*
  Print the group and container hierarchy from *pool* in a tree.

`ct show` [*options*] *id*
  Show all or selected parameters of container *id*.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

`ct start` [*options*] *id*
  Start container *id*.

    `-F`, `--[no-]foreground`
      Open container console (can be later detached), see `ct console`.

`ct stop` [*options*] *id*
  Stop container *id*. By default, `osctld` will send a signal to the container's
  init process to cleanly shutdown and wait until it finishes or *timeout*
  seconds passes. If it time outs, the container is killed. This behaviour can
  be changed with options `--timeout`, `--kill` and `--dont-kill`.

    `-F`, `--[no-]foreground`
      Open container console (can be later detached), see `ct console`.

    `-k`, `--kill`
      Do not request a clean shutdown, kill the container immediately.

    `--dont-kill`
      If the clean shutdown does not finish in *timeout* seconds, exit with
      error, do not kill the container.

    `-t`, `--timeout` *timeout*
      How many seconds to wait for the container to cleanly shutdown before
      killing it or failing, depending on whether option `--dont-kill` is set.
      The default timeout is *60* seconds.

`ct restart` [*options*] *id*
  Restart container *id*. By default, `ct restart` calls `ct stop` and `ct start`
  in succession. Like with `ct stop`, if the container does not cleanly shutdown
  in *timeout* seconds, it is killed. This behaviour can be changed with options
  `--timeout`, `--kill` and `--dont-kill`.

  If option `--reboot` is used, the container's init process is signaled to
  reboot the system. `osctld` has no way of knowing whether the init process
  responds and the reboot actually takes place.

    `-F`, `--[no-]foreground`
      Open container console (can be later detached), see `ct console`.

    `-r`, `--reboot`
      Request a reboot of the container by signaling its init process. 
      If the init process does not respond to the configured signal, nothing
      happens.

    `-k`, `--kill`
      Do not request a clean shutdown, kill the container immediately.

    `--dont-kill`
      If the clean shutdown does not finish in *timeout* seconds, exit with
      error, do not kill the container.

    `-t`, `--timeout` *timeout*
      How many seconds to wait for the container to cleanly shutdown before
      killing it or failing, depending on whether option `--dont-kill` is set.
      The default timeout is *60* seconds.

`ct attach` [*options*] *id*, `ct enter` [*options*] *id*
  Attach container *id* and open a shell. `osctld` tries to open `bash`,
  `busybox` and falls back to `/bin/sh`. The shell is not reading any personal
  configuration files from within the container in order to provide a unified
  shell interface across all containers. Use option `--user-shell` to change
  this behaviour.

    `-u`, `--user-shell`
      Load the shell that's configured in the container's `/etc/passwd` for
      `root` and read personal configuration files, such as `.bashrc`.

`ct console` [*options*] *id*
  Attach container console. Unlike in LXC, this console is persistent across
  container reboots. It can be attached even when the container is stopped.
  The console can be detached using **Ctrl-a q**.

  If global option `-j`, `--json` is set, the console will not manipulate the
  TTY, but instead will accept JSON commands on standard input. Output from the
  console will be written to standard output as-is. To detach the console, send
  `SIGTERM` to `osctl`. To learn more about the JSON commands, see
  `CONSOLE INTERFACE`.

    `-t`, `--tty` *n* - Select which TTY to attach, defaults to **0**.

`ct exec` *id* *cmd...*
  Attach container *id* and execute command *cmd* within a shell.
  stdin/stdout/stderr of *cmd* is piped to your current shell.

`ct set autostart` [*options*] *id*
  Start the container automatically when `osctld` starts or when its pool is
  imported.

    `-p`, `--priority` *n*
      Priority determines container start order. `0` is the highest priority,
      higher number means lower priority. Containers with the same priority
      are ordered by their ids. The default priority is `10`.

    `-d`, `--delay` *n*
      Time in seconds for which `osctld` waits until the next container is
      started. The default is `5` seconds.

`ct unset autostart` *id*
  Do not start the container automatically.

`ct set hostname` *id* *hostname*
  Set container hostname. Depending on distribution, the hostname is configured
  within the container and an entry is added to `/etc/hosts`. The hostname
  is configured on every container start.

`ct unset hostname` *id*
  Unset container hostname. `osctld` will not touch the container's hostname
  anymore.

`ct set dns-resolver` *id* *address...*
  Configure DNS resolvers for container *id*. At least one DNS resolver is
  needed. Given DNS resolvers are written to the container's `/etc/resolv.conf`
  on every start.

  Note that when you assign a bridged veth with DHCP to the container, it will
  override `/etc/resolv.conf` with DNS servers from DHCP server.

`ct unset dns-resolver` *id*
  Unset container DNS resolvers. `osctld` will no longer manipulate the
  container's `/etc/resolv.conf`.

`ct set nesting` *id* `enabled`|`disabled`
  Allow/disallow LXC nesting for container *id*. The container needs to be
  restarted for the change to take effect.

`ct set cpu-limit` *id* *limit*
  Configure CFS bandwidth control cgroup parameters to enforce CPU limit. *limit*
  represents maximum CPU usage in percents, e.g. `100` means the container can
  fully utilize one CPU core.

  This command is just a shortcut to `ct cgparams set`, two parameters are
  configured: `cpu.cfs_period_us` and `cpu.cfs_quota_us`. The quota is calculated
  as: *limit* / 100 \* *period*.

    `-p`, `--period` *period*
      Length of measured period in microseconds, defaults to `100000`,
      i.e. `100 ms`.

`ct unset cpu-limit` *id*
  Unset CPU limit. This command is a shortcut to `ct cgparams unset`.

`ct set memory` *id* *memory* [*swap*]
  Configure the maximum amount of memory and swap the container will be able to
  use. This command is a shortcut to `ct cgparams set`. Memory limit is set
  with cgroup parameter `memory.limit_in_bytes`. If swap limit is given as well,
  parameter `memory.memsw.limit_in_bytes` is set to *memory* + *swap*.

  *memory* and *swap* can be given in bytes, or with an appropriate suffix, i.e.
  `k`, `m`, `g`, or `t`.

`ct unset memory` *id*
  Unset memory limits. This command is a shortcut to `ct cgparams unset`.

`ct chown` *id* *user*
  Move container *id* to user namespace *user*. The container has to be stopped
  first.

`ct chgrp` [*options*] *id* *group*
  Move container *id* to group *group*. The container has to be stopped first.

    `--missing-devices` `check`|`provide`|`remove`
      The container may require access to devices that are not available in the
      target group. This option determines how should `osctld` treat those
      missing devices. `check` means that if a missing device is found, an error
      is returned and the operation is aborted. `provide` will add missing
      devices to the target group and all its parent groups, it will also ensure
      sufficient access mode. `remove` will remove all unavailable devices from
      the container. The default mode is `check`.

`ct passwd` *id* *user* [*password*]
  Change password of *user* in container *id*. The user has to already exist.
  If *password* is not given as an argument, it is prompted for on stdin.
  The container has to be running for this command to work, as it is using
  `passwd` or `chpasswd` from the container's system.

`ct su` *id*
  Switch to the user of container *id* and cd to its LXC home. The shell
  is tailored only for container *id*, do not use it to manipulate any other
  containers, even in the same LXC home. Every container can have a different
  cgroup configuration, which would be broken.

  Also not that when a container is started from this shell using `lxc-start`,
  `ct console` for tty0 will not be functional.

`ct cd` [*options*] *id*
  Opens a new shell with changed current working directory, based on *options*.
  When no option is specified, the directory is changed to the container's
  rootfs. Close the shell to return to your previous session.
  
    `-l`, `--lxc`
      Go to LXC config directory
    
    `-r`, `--runtime`
      Go to */proc/<init_pid>/root*. The container must be running for the path
      to exist.

`ct log cat` *id*
  Write the contents of container *id* log to the stdout.

`ct log path` *id*
  Write the path to the log file of container *id* to stdout.

`ct reconfigure` *id*
  Regenerate LXC configuration.

`ct export` [*options*] *id* *file*
  Export container *id* into a tar archive *file*. The archive will contain
  the container's configuration, its user, group and data. The exported archive
  can later be imported on the same or a different node.

    `--[no-]consistent`
      Enable/disable consistent export. When consistently exporting a running
      container, the container is stopped, so that applications can gracefully
      exit and save their state to disk. Once the export is finished,
      the container is restarted.

    `--compression` *auto* | *off* | *gzip*
      Enable/disable compression of the dumped ZFS data streams. The default is
      *auto*, which uses compressed stream, if the dataset has ZFS compression
      enabled. If the compression is not enabled on the dataset, the stream
      will be compressed using *gzip*. *off* disables compression, but if
      ZFS compression is enabled, the data is dumped as-is. *gzip* enforces
      compression, even if ZFS compression is enabled.

`ct import` [*options*] *file*
  Import a container defined in archive *file*, which can be generated by
  `ct export`.

    `--as-id` *id*
      Import the container and change its id to *id*. Using this option, it is
      possible to import the same *file* multiple times, essentially cloning
      the containers.

    `--as-user` *name*
      Import the container as an existing user *name*. User configuration from
      *file* is not used.

    `--as-group` *name*
      Import the container into an existing group *name*. Group configuration
      from *file* is not used.

    `--dataset` *dataset*
      Use a custom dataset for the container's rootfs. The dataset and all its
      parents are created, if it doesn't already exist.

    `--missing-devices` `check`|`provide`|`remove`
      The imported container may require access to devices that are not configured
      on this system. This option determines how should `osctld` treat those missing
      devices. `check` means that if a missing device is found, an error is returned
      and the import is aborted. `provide` will add missing devices to all parent
      groups and ensure sufficient access mode. `remove` will remove all unconfigured
      devices from the container. The default mode is `check`.

`ct migrate stage` [*options*] *id* *destination*
  Stage migration of container *id* to *destination*. *destination* is a host
  name or an IP address of another vpsAdminOS node. The container's user, group
  and config files are copied over SSH to *destination*.

    `-p`, `--port` *port*
      SSH port, defaults to `22`.

`ct migrate sync` *id*
  Continue staged migration of container *id* to previously configured
  *destination*. During sync, a snapshot of the container's dataset is taken
  and sent to the *destination*.

`ct migrate transfer` *id*
  This command stops the container if it is running, makes another snapshot
  of the container's dataset, sends it to *destination*. The container is then
  started on the *destination* node.

`ct migrate cleanup` [*options*] *id*
  Perform a cleanup after migration of container *id*. The migration state is
  reset and the container is by default deleted.

    `-d`, `--[no-]delete`
      Delete the container from the source node. The default is to delete the
      container.

`ct migrate cancel` [*options*] *id*
  Cancel a migration of container *id*. The migration's state is deleted from
  the source node, and all trace of the container is deleted from the
  *destination* node. This command has to be called in-between migration steps
  up until `ct migrate transfer`, it cannot stop the migration if one of the
  steps is still in progress.

    `-f`, `--force`
      Cancel the migration's state on the local node, even if the remote node
      refuses to cancel. This is helpful when the migration state between the
      two nodes gets out of sync. The remote node may remain in an unconsistent
      state, but from there, the container can be deleted using `osctl ct del`
      if needed.

`ct migrate now` [*options*] *id* *destination*
  Perform a full container migration in a single step. This is equal to running
  `ct migrate stage`, `ct migrate sync`, `ct migrate transfer` and
  `ct migrate cleanup` in succession.

    `-p`, `--port` *port*
      SSH port, defaults to `22`.

    `-d`, `--[no-]delete`
      Delete the container from the source node. The default is to delete the
      container.

`ct monitor` *id*
  Monitor state changes of container *id* and print them on standard output.
  If global option `-j`, `--json` is used, the state changes are reported
  in JSON.

`ct wait` *id* *state...*
  Block until container *id* enters one of given states.

`ct top` [*options*]
  top-like TUI application showing running containers and their CPU, memory,
  BlkIO and network usage. `ct top` can function in two modes: *realtime* and
  *cumulative*. *realtime* mode shows CPU usage in percent and other resources
  as usage per second, except memory and the number of processes. *cumulative*
  mode shows all resource usage accumulated from the time `ct top` was started.

  Key bindings:

  Keys                      | Action
  ------------------------- | -------------
  `q`                       | Quit
  `<`, `>`, *left*, *right* | Change sort column
  `r`, `R`                  | Reverse sort order
  `m`                       | Toggle between `realtime` and `cumulative` mode.
  `?`                       | Show help message.

  When global option `-j`, `--json` is used, the TUI is replaced by JSON
  being periodically printed on standard output. Every line describing resource
  usage at the time of writing. `ct top` with JSON output can be manually
  refreshed by sending it `SIGUSR1`.

    `-r`, `--rate` *n*
      Refresh rate in seconds, defaults to 1 second.

`ct pid` [*pid...*] | `-`
  Find containers by process IDs. By default, the process IDs are passed as
  command-line arguments. If the first PID is `-`, the PIDs are read from
  standard input, one PID per line.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

`ct assets` [*options*] *id*
  List container assets and their state.

    `-v`, `--verbose`
      Show detected errors.

`ct cgparams ls` [*options*] *id* [*parameters...*]
  List cgroup parameters for container *id*. If no *parameters* are provided,
  all configured parameters are listed.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters adn exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output.

    `-S`, `--subsystem` *subsystem*
      Filter by cgroup subsystem, comma separated.

    `-a`, `--all`
      Include parameters from parent groups up to root group.

`ct cgparams set` *id* *parameter* *value...*
  Set cgroup parameter *parameter* of container *id* to *value*. `osctld` will
  make sure this parameter is always set when the container is started. The
  parameter can be for example `cpu.shares` or `memory.limit_in_bytes`. cgroup
  subsystem is derived from the parameter name, you do not need to supply it.

  It is possible to set multiple values for a parameter. The values are written
  to the parameter file one by one. This can be used for example for the
  `devices` cgroup subsystem, where you may need to write to `devices.deny` and
  `devices.allow` multiple times.

    `-a`, `--append`
      Append new values, do not overwrite previously configured values for
      *parameter*.

`ct cgparams unset` *id* *parameter*
  Unset cgroup parameter *parameter* from container *id*. Selected cgroup
  parameters are reset, the rest is left alone and merely removed from `osctld`
  config.

  The following parameters are reset:

  - `cpu.cfs_quota_us`
  - `memory.limit_in_bytes`
  - `memory.memsw.limit_in_bytes`

`ct cgparams apply` *id*
  Apply all cgroup parameters defined for container *id*, its group and all
  its parent groups, all the way up to the root group.

`ct devices ls` [*options*] *id*
  List configured devices.

    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output, comma separated. Defaults to a selected
      subset of available parameters.

`ct devices add` [*options*] *id* `block`|`char` *major* *minor* *mode* [*device*]
  Allow container *id* to use `block`/`char` device identified by the *major*
  and *minor* numbers, see mknod(1) for more information. *mode* determines
  what is the container allowed to do: `r` to read, `w` to write, `m` to call
  `mknod`. For now, unprivileged containers cannot call `mknod`, so allowing it
  here doesn't do anything.

  If *device* is provided, `osctld` will prepare the device node within the
  container's `/dev` during every container start.

  Devices added in this way are always promoted, see `ct devices promote`.

    `-p`, `--[no-]parents`
      The device that is being added has to be provided by all parent groups,
      up to the *root* group. When this switch is enabled, `osctld` will add
      the device to all parent groups that do not already have it.

`ct devices del` *id* `block`|`char` *major* *minor*
  Forbid the container to use specified device and remove its device node, if it
  exists.

`ct devices chmod` [*options*] *id* `block`|`char` *major* *minor* *mode*
  Change the access mode of the specified device to *mode*. The mode can be
  changed only if the container's group provides the device with all necessary
  access modes, or `-p`, `--parents` is used.

  If the device was inherited, it is promoted to a standalone device and saved
  into the config file with the modified access mode.

    `-p`, `--parents`
       Ensure that all parent groups provide the device with the required
       access mode. Parents that do not provide correct access modes are updated.

`ct devices promote` *id* `block`|`char` *major* *minor*
  Promoting a device will ensure that parent groups will have to provide it,
  it is a declaration of an explicit requirement. It will no longer be possible
  to remove the device from parent groups, without explicitly removing it from
  the container as well, e.g. using `group devices del -r`.

  Promoted devices are also exported together with the container, and on import,
  the target vpsAdminOS has to provide these devices as well, or forcefully
  remove them, see `ct export` and `ct import`.

  When migrating containers with promoted devices, the target node has to
  provide those devices, otherwise the migration process will fail in the first
  step, i.e. `ct migrate stage`.

`ct devices inherit` *id* `block`|`char` *major* *minor*
  Inherit the device from the parent group. This removes the explicit requirement
  on the pecified device, i.e. reverses `ct devices promote`. The access mode,
  if different from the group, will revert to the acess mode defined by the
  parent group.

  Note that if the parent group does not have the device set as inheritable,
  it will be removed from the container.

`ct prlimits ls` *id* [*limits...*]
  List configured resource limits. If no *limits* are provided, all configured
  limits are listed.

    `-H`, `--hide-header`
        Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters adn exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

`ct prlimits set` *id* *limit* *soft\_and\_hard*, `ct prlimits set` *id* *limit* *soft* *hard*
  Set resource *limit* on container *id*. Limit names and their descriptions
  can be found in setrlimit(2). The permitted names are the "RLIMIT\_" resource
  names in lowercase without the "RLIMIT\_" prefix, eg. `RLIMIT_NOFILE` should
  be specified as **nofile**.

  If *hard* is not provided, it equals to the *soft* limit. The value can be
  either an integer or **unlimited**.

`ct prlimits unset` *id* *limit*
  Unset resource *limit* from container *id*.

`ct netif ls` [*options*] *id*
  List configured network interfaces for container *id*.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

    `-l`, `--link` *bridge*
      Filter by linked bridge.

    `-t`, `--type` *type*
      Filter by interface type (`bridge` or `routed`)

`ct netif new bridge` [*options*] `--link` *bridge* *id* *name*
  Create a new bridge network interface in container *id*. This will create
  a pair of veth interfaces, one on the host, one in the container. The veth
  on the host will be linked to *bridge*. The interface will appear as *name*
  within the container. *bridge* is not managed by `osctld`, it must be provided
  by the system administrator in advance.

  Currently, the container uses DHCP to configure the interface.
  The container has to be stopped for this command to be allowed.

    `--link` *bridge*
      What bridge should the interface be linked with, required.

    `--hwaddr` *addr*
      Set a custom MAC address. Every **x** in the address is replaced by
      a random value. By default, the address is dynamically allocated.

`ct netif new routed` [*options*] `--via` *network* *id* *name*
  Create a new routed network interface in container *id*. Like for **bridge**
  interface, a pair veth is created. The difference is that the veth is not part
  of any bridge. Instead, IP addresses are routed to the container via
  an interconnecting *network*. `osctld` will automatically setup appropriate
  routes on the host veth interface. The interface will appear as *name* within
  the container.
  
  The container has to be stopped for this command to be allowed.

    `--via` *network*
      Route via network, required. Can be used once for IPv4 and once for IPv6,
      depending what addresses you want to be able to route.

    `--hwaddr` *addr*
      Set a custom MAC address. Every **x** in the address is replaced by
      a random value. By default, the address is dynamically allocated.

`ct netif del` *id* *name*
  Remove interface *name* from container *id*.
  The container has to be stopped for this command to be allowed.

`ct netif rename` *id* *old-name* *new-name*
  Rename network interface. The container has to be stopped for this operation
  to pass.

`ct netif ip add` *id* *name* *addr*
  Add IP address *addr* to interface *name* of container *id*. `osctld` will
  setup routing in case of **routed** interface and add the IP address to the
  container's interface.

`ct netif ip del` *id* *name* *addr*
  Remove IP address *addr* from interface *name* of container *id*.

`ct netif ip ls` *id* *name*
  List IP addresses assigned to interface *name* of container *id*.

`ct dataset ls` [*options*] *id* [*properties...*]
  List datasets of container *id*. *properties* is a space separated list of
  ZFS properties to read and print. Listed dataset names are relative to the
  container's root dataset, the root dataset itself is called `/`.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

`ct dataset new` [*options*] *id* *name* [*mountpoint*]
  Create a new subdataset of the container's root dataset. The *name*
  is relative to the container's root dataset, e.g. in the default configuration,
  `osctl ct dataset new <id> var` will create ZFS dataset `<pool>/ct/<id>/var`
  and mount it to directory `/var` within the container. The target *mountpoint*
  can be optionally provided as an argument.

  Non-existing dataset parents are automatically created and mounted with respect
  to `--[no-]mount`.
  
  Created datasets are automatically shifted into the container's user namespace.
  Of course, container datasets can be managed using `zfs` directly, but you'd
  have to set `uidoffset` and `gidoffset` properties manually.

  Datasets should be mounted using `ct mounts dataset`, mounts created with
  `ct mounts new` might not survive container export/import on different
  configurations.

    `--[no-]mount`
      Mount created datasets to the container, under the mountpoint of its
      parents or `/`. Created datasets are mounted to the container by default.

`ct dataset del` [*options*] *id* *name*
  Delete container subdataset *name*. The root dataset cannot be deleted.

    `-r`, `--recursive`
      Recursively delete all children as well. Disabled by default.

    `-u`, `--[no-]umount`, `--[no-]unmount`
      Unmount selected dataset and all its children when in recursive mode
      before the deletion. By default, mounted datasets will abort the deletion.

`ct mounts ls` *id*
  List mounts of container *id*.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output.

`ct mounts new` *options* *id*
  Create a new mount for container *id* described by *options*. The *fs* is not
  mounted immediately, but the next time the container starts.

    `--fs` *fs*
      File system or device to mount, required.

    `--mountpoint` *mountpount*
      Mountpoint within the container, required.

    `--type` *type*
      File system type, required.

    `--opts` *opts*
      Options, required. Standard mount options depending on the filesystem
      type, with two extra options from LXC: `create=file` and `create=dir`.

    `--[no-]automount`
      Activate this mount when the container starts. Enabled by default.

`ct mounts dataset` *options* *id* *name* *mountpoint*
  Mount subdataset *name* into container *id*. Only subdatasets of container
  *id* can be mounted in this way. Dataset mounts can survive container
  export/import or migration to a host with different configuration. Mounts
  created via `ct mounts new` have a fixed *fs* path, which would change
  on a host with a zpool named differently and the container would refuse to
  start.

  In addition, `ct mounts dataset` does not mount the top-level directory, but
  rather a subdirectory called `private`. This prevents the container to access
  the `.zfs` special directory, which could be used to create or destroy
  snapshots from within the container.

    `--ro`, `--read-only`
      Mount the dataset in read-only mode.

    `--rw`, `--read-write`
      Mount the dataset in read-write mode. This is the default.

    `--[no-]automount`
      Activate this mount when the container starts. Enabled by default.

`ct mounts register` [*options*] *id* *mountpoint*
  Register a manually created mount. This can be used to register mounts created
  in `pre-mount` or `post-mount` script hooks (see `SCRIPT HOOKS`) or any other
  mount within the container that you wish to control. All options are optional,
  but unless you provide *fs* and *type*, you won't be able to use command
  `ct mounts activate`.

  `ct mounts register` works only on starting or running container. All mounts
  registered using this command will be forgotten once the container is stopped.

    `--fs` *fs*
      File system or device to mount.

    `--type` *type*
      File system type.

    `--opts` *opts*
      Mount options. Standard mount options depending on the filesystem
      type, with two extra options from LXC: `create=file` and `create=dir`.

    `--on-ct-start`
      Use this option if you're calling `ct mounts register` from script hooks,
      see `SCRIPT HOOKS`. Without this option, calling this command from hook
      scripts will cause a deadlock -- the container won't start and `osctld`
      will be tainted as well.

`ct mounts activate` *id* *mountpoint*
  Mount the directory inside the container. The container has to be running.
  Note that this command will mount the directory multiple times if called
  when the directory is already mounted.

`ct mounts deactivate` *id* *mountpoint*
  Unmount the directory from the running container.

`ct mounts del` *id* *mountpoint*
  Remove *mountpoint* from container *id*. The *mountpoint* is not unmounted
  immediatelly, but when the container stops.

`group new` *options* *name*
  Create a new group for resource management.

    `--pool` *name*
      Pool name, optional.

    `-p`, `--parents`
      Create all missing parent groups.

    `--cgparam` *parameter*=*value*
      Set cgroup parameter, may be used more than once. See `group cgparams set`
      for what the parameter is.

`group del` *name*
  Delete group *name*. The group musn't be used by any container.

`group ls` [*options*] [*names...*]
  List available groups. If no *names* are provided, all groups are listed.
    
    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output, comma separated. Defaults to a selected
      subset of available parameters.

    `--pool` *names*
      Filter by pool, comma separated.

`group tree` *pool*
  Print the group hierarchy from *pool* in a tree.

`group show` *name*
  Show group info.

`group cgparams ls` [*options*] *name* [*parameters...*]
  List cgroup parameters for group *name*. If no *parameters* are provided,
  all configured parameters are listed.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters adn exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output.

    `-S`, `--subsystem` *subsystem*
      Filter by cgroup subsystem, comma separated.

    `-a`, `--all`
      Include parameters from parent groups up to root group.

`group cgparams set` *name* *parameter* *value...*
  Set cgroup parameter *parameter* of group *name* to *value*. `osctld` will
  make sure this parameter is always set when the container is started. The
  parameter can be for example `cpu.shares` or `memory.limit_in_bytes`. cgroup
  subsystem is derived from the parameter name, you do not need to supply it.

  It is possible to set multiple values for a parameter. The values are written
  to the parameter file one by one. This can be used for example for the
  `devices` cgroup subsystem, where you may need to write to `devices.deny` and
  `devices.allow` multiple times.

    `-a`, `--append`
      Append new values, do not overwrite previously configured values for
      *parameter*.

`group cgparams unset` *name* *parameter*
  Unset cgroup parameter *parameter* from group *name*. Selected cgroup
  parameters are reset, the rest is left alone and merely removed from `osctld`
  config.

  The following parameters are reset:

  - `cpu.cfs_quota_us`
  - `memory.limit_in_bytes`
  - `memory.memsw.limit_in_bytes`

`group cgparams apply` *name*
  Apply all cgroup parameters defined for group *name* and all its parent
  groups, all the way up to the root group.

`group devices ls` [*options*] *id*
  List configured devices.

    `-H`, `--hide-header`
      Do not show header, useful for scripts.

    `-L`, `--list`
      List available parameters and exit.

    `-o`, `--output` *parameters*
      Select parameters to output, comma separated. Defaults to a selected
      subset of available parameters.

`group devices add` [*options*] *id* `block`|`char` *major* *minor* *mode* [*device*]
  Allow container *id* to use `block`/`char` device identified by the *major*
  and *minor* numbers, see mknod(1) for more information. *mode* determines
  what is the container allowed to do: `r` to read, `w` to write, `m` to call
  `mknod`. For now, unprivileged containers cannot call `mknod`, so allowing it
  here doesn't do anything.

  If *device* is provided, `osctld` will prepare the device node within the
  container's `/dev` during every container start.

    `-i`, `--[no-]inherit`
      Determines whether child groups and containers should inherit the device,
      i.e. be allowed to use it with the same access *mode*.

    `-p`, `--[no-]parents`
      The device that is being added has to be provided by all parent groups,
      up to the *root* group. When this switch is enabled, `osctld` will add
      the device to all parent groups that do not already have it.

`group devices del` *id* `block`|`char` *major* *minor*
  Forbid the container to use specified device and remove its device node, if it
  exists. If the device is used by any descendant groups or containers, it can be
  deleted only with the `-r`, `--recursive` switch.

    `-r`, `--recursive`
      Delete the device from all child groups and containers.

`group devices chmod` [*options*] *id* `block`|`char` *major* *minor* *mode*
  Change the access mode of the specified device to *mode*. The mode can be
  changed only if all parent groups provide the device with all necessary
  access modes and if no child group or container has broader access mode
  requirements. Use `-p`, `--parents` or `-r`, `--recursive` to override
  parent or child groups and containers.

  If the device was inherited, it is promoted to a standalone device and saved
  into the config file with the modified access mode.

    `-p`, `--parents`
      Ensure that all parent groups provide the device with the required
      access mode. Parent groups that do not provide correct access modes
      are updated and the missing access modes are set.

    `-r`, `--recursive`
      Change the access mode of all child groups and containers.

`group devices promote` *name* `block`|`char` *major* *minor*
  Promoting a device will ensure that parent groups will have to provide it,
  it is a declaration of an explicit requirement. It will no longer be possible
  to remove the device from parent groups, without explicitly removing it from
  this group as well, e.g. using `group devices del -r`.

`group devices inherit` *name* `block`|`char` *major* *minor*
  Inherit the device from the parent group. This removes the explicit requirement
  on the pecified device, i.e. reverses `ct devices promote`. The access mode,
  if different from the parent, will revert to the acess mode defined by the
  parent group.

  Note that if the parent group does not have the device set as inheritable,
  the device will be removed. This command cannot be used for the `root` group,
  as it has no parent to inherit from.

`group devices set inherit` *name* `block`|`char` *major* *minor*
  Set specified device as inheritable. Child groups and container will inherit
  this device immediately.

`group devices unset inherit` *name* `block`|`char` *major* *minor*
  Prevent the specified device from being automatically inherited by child
  groups and containers. The device is immediately removed from all child groups
  and containers, that have previously inherited it. Promoted devices are left
  alone.

`group set cpu-limit` *name* *limit*
  Configure CFS bandwidth control cgroup parameters to enforce CPU limit. *limit*
  represents maximum CPU usage in percents, e.g. `100` means the container can
  fully utilize one CPU core.

  This command is just a shortcut to `group cgparams set`, two parameters are
  configured: `cpu.cfs_period_us` and `cpu.cfs_quota_us`. The quota is calculated
  as: *limit* / 100 \* *period*.

    `-p`, `--period` *period*
      Length of measured period in microseconds, defaults to `100000`,
      i.e. `100 ms`.

`group unset cpu-limit` *name*
  Unset CPU limit. This command is a shortcut to `group cgparams unset`.

`group set memory` *name* *memory* [*swap*]
  Configure the maximum amount of memory and swap the group will be able to
  use. This command is a shortcut to `group cgparams set`. Memory limit is set
  with cgroup parameter `memory.limit_in_bytes`. If swap limit is given as well,
  parameter `memory.memsw.limit_in_bytes` is set to *memory* + *swap*.

  *memory* and *swap* can be given in bytes, or with an appropriate suffix, i.e.
  `k`, `m`, `g`, or `t`.

`group unset memory` *name*
  Unset memory limits.

`group assets` [*options*] *name*
  List group's assets (datasets, files, directories) and their state.

    `-v`, `--verbose`
      Show detected errors.

`migration key gen` [*options*]
  Generate public/private key pair that is used when migrating containers to
  other vpsAdminOS nodes.

    `-t`, `--type` `rsa` | `ecdsa` | `ed25519`
      Key type, defaults to `rsa`.

    `-b`, `--bits` *bits*
      Specifies the number of bits in the key to create. Defaults to `4096` for
      `rsa` and `591` for `ecdsa`.

    `-f`, `--force`
      Overwrite the keys if they already exist.

`migration key path` [`public` | `private`]
  Print the path to either the `public` or `private` key. Defaults to the
  `public` key.

`migration authorized-keys ls`
  List keys that are authorized to migrate containers to this node.

`migration authorized-keys add`
  Authorize given key to migrate containers to this node. The key is read from
  the standard input and must be provided on a single line.

`migration authorized-keys del` *index*
  Remove the key identified by its *index*, which can be obtained by
  `migration authorized-keys ls`.

`repository ls`
  List configured repositories, from which container templates are downloaded.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters adn exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output.

`repository add` *name* *url*
  Add repository at *url* as *name* to the default or the selected pool.
  The pool can be selected using global option `--pool`.

`repository del` *name*
  Remove repository *name* from the default or the selected pool.

`repository enable` *name*
  Enable repository *name*. Enabled repositories are searched for templates
  when creating containers. Repositories are enabled by default upon addition.

`repository disable` *name*
  Disable repository *name*. Disabled repositories are not searched for templates,
  until reenabled.

`repository assets` *name*
  Show repository's assets and their state.

`repository templates ls` [*option*] *name*
  List templates available in repository *name*.

    `-H`, `--hide-header`
      Do not show header, useful for scripting.

    `-L`, `--list`
      List available parameters adn exit.
    
    `-o`, `--output` *parameters*
      Select parameters to output.

    `--vendor` *vendor*
      Filter by vendor.

    `--variant` *variant*
      Filter by variant.

    `--arch` *arch*
      Filter by architecture.

    `--distribution` *distribution*
      Filter by distribution.

    `--version` *version*
      Filter by distribution version.

    `--tag` *tag*
      Filter by version tag.

    `--cached`
      Show only locally cached templates.

    `--uncached`
      Show only locally uncached templates.

`monitor`
  Print all events reported by `osctld` to standard output. If global option
  `-j`, `--json` is used, the events are printed in JSON.

`history` [*pool...*]
  Print management history of all or selected pools. If global option
  `-j`, `--json` is used, the events are printed in JSON.

`assets` [*options*]
  List `osctld` assets (datasets, files, directories) and their state.

    `-v`, `--verbose`
      Show detected errors.

`healthcheck` [*options*] [*pool...*]
  Verify `osctld` assets and optionally also assets of selected pools, which
  include all user, group and container assets stored on selected pools.

    `-a`, `--all`
      Verify all pools.

`shutdown`
  Export all pools and stop all containers. This command should be used at
  system shutdown. Since all pools are immediately disabled, no container can be
  started. All running containers are stopped. System users and groups are left
  alone. This action can be reversed by reimporting selected pools.

`help` [*command...*]
  Shows a list of commands or help for one command

## TEMPLATE NAMES
Template can either be a tar archive with the extension **tar.gz**, or a ZFS
stream file with extension **dat.gz**. The name then has to match the following
format: <*distribution*>-<*version*>\*.<*extension*>. *distribution* is
a distribution name in lower case, e.g. `alpine`, `centos` or `debian`.
*version* is the distribution release version, e.g. `3.6` for `alpine`,
`7.0` for `centos` or `9.0` for `debian`.

## SCRIPT HOOKS
`osctld` can execute user-defined scripts when containers are being started
or stopped. Script hooks are located at `/<pool>/hook/ct/<ctid>/<hook>`, use
`ct assets` to get the exact path for your container. In order for script hooks
to be called, they need to be executable. All script hooks are run as `root`
on the host, but mount namespace may differ, see below.

Note that many `osctl` commands called from script hooks will not work as expected
and may cause deadlocks. The hooks are run when the container is locked within
`osctld`, so if another `osctl` process called from a hook needs the lock,
a deadlock occurs. You should avoid calling `osctl` from hooks.

`pre-start`
  `pre-start` hook is run in the host's namespace before the container is mounted.
  The container's cgroups have already been configured and distribution-support
  code has been run. If `pre-start` exits with a non-zero status, the container's
  start is aborted.

`veth-up`
  `veth-up` hook is run in the host's namespace when the veth pair is created.
  Names of created veth interfaces are available in environment variables
  `OSCTL_HOST_VETH` and `OSCTL_CT_VETH`. If `veth-up` exits with a non-zero
  status, the container's start is aborted.

`pre-mount`
  `pre-mount` is run in the container's mount namespace, before its rootfs is
  mounted. The path to the container's runtime rootfs is in environment variable
  `OSCTL_CT_ROOTFS_MOUNT`. If `pre-mount` exits with a non-zero status, the
  container's start is aborted.

`post-mount`
  `post-mount` is run in the container's mount namespace, after its rootfs
  and all LXC mount entries are mounted. The path to the container's runtime
  rootfs is in environment variable `OSCTL_CT_ROOTFS_MOUNT`. If `post-mount`
  exits with a non-zero status, the container's start is aborted.

`on-start`
  `on-start` is run in the host's namespace, after the container has been
  mounted and right before its init process is executed. If `on-start` exits
  with a non-zero status, the container's start is aborted.

`post-start`
  `post-start` is run in the host's namespace after the container entered state
  `running`. The container's init PID is passed in environment varible
  `OSCTL_CT_INIT_PID`. The script hook's exit status is not evaluated.

`pre-stop`
  `pre-stop` hook is run in the host's namespace when the container is being
  stopped using `ct stop`. If `pre-stop` exits with a non-zero exit status,
  the container will not be stopped. This hook is not called when the container
  is shutdown from the inside.

`on-stop`
  `on-stop` is run in the host's namespace when the container enters state
  `stopping`. The hook's exit status is not evaluated.

`veth-down`
  `veth-down` hook is run in the host's namespace when the veth pair is removed.
  Names of the removed veth interfaces are available in environment variables
  `OSCTL_HOST_VETH` and `OSCTL_CT_VETH`. The hook's exit status is not
  evaluated.

`post-stop`
  `post-stop` is run in the host's namespace when the container enters state
  `stopped`. The hook's exit status is not evaluated.

## Environment variables
All hooks have the following environment variables set:

- `OSCTL_HOOK_NAME`
- `OSCTL_POOL_NAME`
- `OSCTL_CT_ID`
- `OSCTL_CT_USER`
- `OSCTL_CT_GROUP`
- `OSCTL_CT_DATASET`
- `OSCTL_CT_ROOTFS`
- `OSCTL_CT_LXC_PATH`
- `OSCTL_CT_LXC_DIR`
- `OSCTL_CT_CGROUP_PATH`
- `OSCTL_CT_DISTRIBUTION`
- `OSCTL_CT_VERSION`
- `OSCTL_CT_HOSTNAME`
- `OSCTL_CT_LOG_FILE`

## CONSOLE INTERFACE
`osctl --json ct console` accepts JSON commands on standard input. Commands
are separated by line breaks (`\n`). Each JSON command can contain the following
values:

```
{
  "keys": base64 encoded data,
  "rows": number of terminal rows,
  "cols": number of terminal columns
}
```

`keys` is the data to be written to the console. `rows` and `cols` control
terminal size. Example commands:

```
{"keys": "Cg=="}\n
{"keys": "Cg==", "rows": 25, "cols": 80}\n
{"rows": 50, "cols": 120}\n
```

Where `Cg==` is `\n` (enter/return key) encoded in Base64. All values
are optional, but `rows` and `cols` have to be together and empty command
doesn't do anything.

## BUGS
Report bugs to https://github.com/vpsfreecz/vpsadminos/issues.

## ABOUT
`osctl` is a part of [vpsAdminOS](https://github.com/vpsfreecz/vpsadminos).
